/*
 * sysmask.h - definitions for sysmask implementation
 *
 * Copyright (c) 2005 Gang Xiao, GNU-GPL
 *
 */


#ifndef _SYSMASK_H
#define _SYSMASK_H

#include <linux/limits.h>
#include <linux/net.h>

#define SM1_ROPEN	0
#define SM1_WOPEN	0
#define SM1_SOCKOPEN	0
#define SM1_SOCKNET	0
#define SM1_SOCKOP	0
#define SM1_PATH	0
#define SM1_EXEC	0
#define SM1_EXECVE	0

#define SM1_DEV		0
#define SM1_PROCFS	0
#define SM1_CREAT	0
#define SM1_SUID	0
#define SM1_DENYID	0
#define SM1_ORPHAN	0
#define SM1_KILLSMKD	0
#define SM1_IPCCREAT	0

#define SM1_UMASK1	0
#define SM1_UMASK2	0
#define SM1_UMASK3	0
#define SM1_UMASK4	0
#define SM1_LOG		0
#define SM1_LDT		0
#define SM1_DATAEXEC	0
#define SM1_KMOD	0


#define SM1_HARMLESS	(1<<0)
#define SM1_FNOHARM	(1<<1)
#define SM1_POLL	(1<<2)	/* select(), poll(), epoll() */
#define SM1_FMISC	(1<<3)
#define SM1_DREAD	(1<<4)
#define SM1_CHDIR	(1<<5)
#define SM1_OPEN	(1<<6)	/* open() */
#define SM1_CHMOD	(1<<7)

#define SM1_ERASE	(1<<8)	/* creat() etc. */
#define SM1_MMAP	(1<<9)
#define SM1_BRK		(1<<10)	/* brk(), sbrk() */
#define SM1_DIR		(1<<11)
#define SM1_LINK	(1<<12)	/* link(), symlink() */
#define SM1_SOCKETCALL	(1<<13)
#define SM1_FS		(1<<14)
#define SM1_MISC	(1<<15)

#define SM1_SPID	(1<<16)
#define SM1_IPC		(1<<17)
#define SM1_IOCTL	(1<<18)
#define SM1_SIGNAL	(1<<19)
#define SM1_KILL	(1<<20)
#define SM1_FORK	(1<<21)	/* fork(), vfork(), clone() */
#define SM1_RLIM	(1<<22)
#define SM1_SYSMASK	(1<<23)

#define SM1_RT		(1<<24)
#define SM1_MOUNT	(1<<25)
#define SM1_CHROOT	(1<<26)
#define SM1_ROOTMISC	(1<<27)
#define SM1_SYSADM	(1<<28)
#define SM1_NONSTD	(1<<29)
#define SM1_OBSOLETE	(1<<30)
#define SM1_NEW		(1<<31)

#define SM1_ALL		0xFFFFFFFF

#define SM1_FOPEN	(SM1_OPEN|SM1_ERASE|SM1_MMAP|SM1_IPC)
#define SM1_OPEN2	0
#define SM1_DWRITE	(SM1_ERASE|SM1_DIR|SM1_LINK|SM1_MOUNT)
#define SM1_FILE	(SM1_FOPEN|SM1_DREAD|SM1_DWRITE|SM1_CHMOD|SM1_CHDIR|SM1_MMAP|SM1_FS|SM1_FMISC)
#define SM1_ROOT	(SM1_SYSADM|SM1_ROOTMISC|SM1_MOUNT|SM1_CHROOT)
#define SM1_PROC	(SM1_FORK|SM1_KILL|SM1_SIGNAL)
#define SM1_PRIV	(SM1_RLIM|SM1_SYSMASK)
#define SM1_MINIMAL	(SM1_OBSOLETE|SM1_NEW)
#define SM1_NONFREQ	(SM1_MISC|SM1_ROOT|SM1_NONSTD|SM1_OBSOLETE|SM1_NEW)
#define SM1_RESOURCE	(SM1_IPC|SM1_SOCKETCALL|SM1_DIR|SM1_LINK|SM1_PROC|SM1_RT|SM1_SPID|SM1_FS)
#define SM1_STARTUP	SM1_MMAP	/* calls usually only used at program startup */
#define SM1_UMASKS	0

#define SM2_ROPEN	(1<<0)	/* read open checked by smkd */
#define SM2_WOPEN	(1<<1)	/* write open checked by smkd */
#define SM2_SOCKOPEN	(1<<2)	/* socket() */
#define SM2_SOCKNET	(1<<3)	/* no socket() except for unix */
#define SM2_SOCKOP	(1<<4)	/* selective */
#define SM2_PATH	(1<<5)	/* check each pathname */
#define SM2_EXEC	(1<<6)	/* check execution by smkd */
#define SM2_EXECVE	(1<<7)	/* hard mask of execve() */

#define SM2_DEV		(1<<8)	/* deny dev opens except /dev/null. */
#define SM2_PROCFS	(1<<9)	/* access to /proc */
#define SM2_CREAT	(1<<10)	/* create non-existing file */
#define SM2_SUID	(1<<11)	/* Silently ignores suid request unless euid=root */
#define SM2_DENYID	(1<<12)	/* prohibit access to file belong to smk-deny */
#define SM2_ORPHAN	(1<<13)	/* children must exit with parent or get killed */
#define SM2_KILLSMKD	(1<<14)	/* can kill daemon; must be masked for general processes */
#define SM2_IPCCREAT	(1<<15)	/* create ipc objets: not yet implemented */

#define SM2_UMASK1	(1<<16)	/* user mask for interactive reactions */
#define SM2_UMASK2	(1<<17)	/* user mask for interactive reactions */
#define SM2_UMASK3	(1<<18)	/* user mask for interactive reactions */
#define SM2_UMASK4	(1<<19)	/* user mask for interactive reactions */
#define SM2_LOG		(1<<20)	/* log access refusals */
#define SM2_LDT		(1<<21)	/* modify_ldt() */
/* DATAEXEC might be absorbed by LDT */
#define SM2_DATAEXEC	(1<<22)	/* Reserved for future use */
#define SM2_KMOD	(1<<23)	/* block usermodehelper() */


#define SM2_HARMLESS	0
#define SM2_FNOHARM	0
#define SM2_POLL	0
#define SM2_FMISC	0
#define SM2_DREAD	0
#define SM2_CHDIR	0
#define SM2_OPEN	(SM2_ROPEN)
#define SM2_CHMOD	0

#define SM2_ERASE	0
#define SM2_MMAP	0
#define SM2_BRK		0
#define SM2_DIR		0
#define SM2_LINK	0
#define SM2_SOCKETCALL	(SM2_SOCKOPEN|SM2_SOCKNET|SM2_SOCKOP)
#define SM2_FS		0
#define SM2_MISC	0

#define SM2_SPID	0
#define SM2_IPC		(SM2_IPCCREAT)
#define SM2_IOCTL	0
#define SM2_SIGNAL	0
#define SM2_KILL	0
#define SM2_FORK	0
#define SM2_RLIM	0
#define SM2_SYSMASK	0

#define SM2_RT		0
#define SM2_MOUNT	0
#define SM2_CHROOT	0
#define SM2_ROOTMISC	0
#define SM2_SYSADM	0
#define SM2_NONSTD	0
#define SM2_OBSOLETE	0
#define SM2_NEW		0


#define SM2_ALL		0xFFFFFFFF

#define SM2_FOPEN	(SM2_ROPEN|SM2_WOPEN|SM2_DEV|SM2_PROCFS)
#define SM2_OPEN2	(SM2_ROPEN|SM2_WOPEN)
#define SM2_DWRITE	(SM2_CREAT)
#define SM2_FILE	(SM2_FOPEN)
#define SM2_ROOT	(SM2_SUID)
#define SM2_PROC	(SM2_EXEC)
#define SM2_PRIV	(SM2_RLIM)
#define SM2_MINIMAL	(SM2_KILLSMKD)
#define SM2_NONFREQ	(SM2_ROOT|SM2_KILLSMKD|SM2_LDT)
#define SM2_RESOURCE	(SM2_EXECVE|SM2_SUID|SM2_IOCTL|SM2_DEV|SM2_PROCFS)
#define SM2_STARTUP	(SM2_LDT)
#define SM2_UMASKS	(SM2_UMASK1|SM2_UMASK2|SM2_UMASK3|SM2_UMASK4)

	/* Special trigger values */
#define SMT_SOCKET	10000
#define SMT_IPC		11000	/* split triggers not yet implemented */
#define SMT_TICK	20000	/* for tick only */

#define SMKD_EXCEPT	65535

#define SMKD_REQ_OPEN	1
#define SMKD_REQ_EXEC	2
#define SMKD_REQ_SOCKET 3
#define SMKD_REQ_TOKEN	4
#define SMKD_REQ_LOG	5
#define SMKD_REQ_CALL	6
#define SMKD_REQ_TRIGGER 7
#define SMKD_REQ_ISCHROOT	0x00000100

	/* bits for smk_got.ind */
#define SMKD_ANS_NEWMASK	(1<<16)
#define SMKD_ANS_NEWUID		(1<<17)
#define SMKD_ANS_NEWGID		(1<<18)
#define SMKD_ANS_ADDMASK	(1<<19)
#define SMKD_ANS_CHECKUID	(1<<20)
#define SMKD_ANS_NOHLINK	(1<<21)
#define SMKD_ANS_NOSLINK	(1<<22)
#define SMKD_ANS_LOGGED		(1<<23)

#define SMKD_ANS_FOLLOW		(1<<24)
#define SMKD_ANS_TICK		(1<<25)
#define SMKD_ANS_HASDOTDOT	(1<<26)

#define SYSMASK_VERSION 1001
	/* a very arbitrary limit; but performance is sensitive to this. */
#define SMK_PATH_MAX	899
	/* Limit of number of logs for one process */
#define SMK_LOG_LIM	500

	/* syscall request numbers */
#define SMK_VERSION	0
#define SMK_GETMASK	1
#define SMK_SETMASK	2
#define SMK_TRIGGER	3
#define SMK_RESERVE1	4
#define SMK_SMKDREQ	5
#define SMK_SMKDREP	6
#define SMK_SMKDREG	7
#define SMK_GETMASK2	8
#define SMK_SETMASK2	9

#define sm1_masked(x)	(current->smk.active.m1&(x))!=0
#define sm2_masked(x)	(current->smk.active.m2&(x))!=0

struct smkd_request {
	pid_t pid;
	uid_t uid;
	int flags;
	unsigned long sm1, sm2;
	int syscall;
	unsigned long token;
	unsigned long reqtype;
	int reqlen;
	char fname[SMK_PATH_MAX+1];	/* this field must be at the end of structure */
};

struct sysmask_maskset {
	unsigned long m1, m2, cap;
};

#define SMK_EXCEPTS 4
struct sysmask_struct {
	struct sysmask_maskset active, triggered;
	signed short trigger, tcnt;
	unsigned long token;
	unsigned short except[SMK_EXCEPTS];
	unsigned short reserve[4];
};

#define SMK_RLIMS 8
struct sysmask_rlim {
	int type;
	long value;
};

	/* answering structure of smkd */
struct smkd_answer {
	struct sysmask_struct smk;
	int ind, opt;
	uid_t uid;
	gid_t gid;
};

#ifdef __KERNEL__

#include <linux/dcache.h>

#ifndef _SYSMASK_C
extern pid_t smk_daemon_pid;
extern struct task_struct *smk_daemon_task;
extern uid_t sysmask_denyid;
#endif

int sys_checkmask(int syscall);
long sys_sysmask(long req, void *smk, long buflen);
void smk_trigger(int t);
long smk_checkpath(const char *fname);
long smk_recheckpath(struct nameidata *nd);
long smk_checkexec(const char *fname);
long smk_checkexec2(struct file *file);
long smk_checkopen(const char *fname, int *flag);
long smk_checksymlink(const char *from, const char *to);
long smk_openmode(struct nameidata *nd, int flag);
long smk_checksocket(struct socket *soc, int type, struct sockaddr __user *addr, int addrlen);
long smk_check_end(void);
#endif /* __KERNEL__ */

#define __NR_sysmask	56

#define getsysmask(x) sysmask(SMK_GETMASK,x,sizeof(struct sysmask_struct))
#define getsysmask2(x) sysmask(SMK_GETMASK2,x,sizeof(struct sysmask_struct))
#define setsysmask(x) sysmask(SMK_SETMASK,x,sizeof(struct sysmask_struct))

#endif /* _SYSMASK_H */

